/*

   John's peano curve generator
   
   cmdline takes args:
    filename options

*/
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <graphics.h>

/* global types and vars */

#define MAX 50
#define BIG 30000

typedef struct seg {
  float x1,y1,x2,y2;
} seg;

typedef struct fig {
  float x1,y1,x2,y2;
  short n1,n2;      /* -1 means last level */
  short t;          /* type= d,r,b */
  short ys;         /* + or - */
} fig;

struct vconfig vc;
int maxd = 4,drall=0,minp = 5;
fig p[MAX]; 
int n;
float yF=1.0;  /* yfactor */

#define drawseg(t,f) \
  pen_color(f); \
  move_to(t.x1,t.y1); \
  line_to(t.x2,t.y2)

/*
  This is a recursive function to draw peano curves, the args are
    a - the baseline segment for this level of recursion
    p - the array of segments making the repeatable figure (normalized)
    d - the max depth of each segment in p
    n - number of segments in p
    c - current depth

    if U is the vector form of A, and V is perpendicular to U,
    the segment T = A + px U + py V.
*/
void peano(seg *a, int c,float yf)
{
  seg t;
  int i,sz;
  float dx,dy;

  for (i=0; i<n; ++i) 
    if (c<=p[i].n2 && c>=p[i].n1 || p[i].n1==BIG && c==maxd) {
      dx = a->x2-a->x1;
      dy = a->y2-a->y1;
      if (p[i].ys=='-' && c&1) {
        t.x1 = a->x1 + p[i].x1*dx - p[i].y1*dy*yf;
        t.y1 = a->y1 + p[i].x1*dy + p[i].y1*dx*yf;
        t.x2 = a->x1 + p[i].x2*dx - p[i].y2*dy*yf;
        t.y2 = a->y1 + p[i].x2*dy + p[i].y2*dx*yf;
      } else {
        t.x1 = a->x1 + p[i].x1*dx + p[i].y1*dy*yf;
        t.y1 = a->y1 + p[i].x1*dy - p[i].y1*dx*yf;
        t.x2 = a->x1 + p[i].x2*dx + p[i].y2*dy*yf;
        t.y2 = a->y1 + p[i].x2*dy - p[i].y2*dx*yf;
      }
      if (p[i].t=='f') {
        pen_color(c+1);
        _floodfill(t.x1,t.y1,c+1); continue; }
      sz = dx<0 ? -dx : dx;        /* new york distance formula */
      sz+= dy<0 ? -dy : dy;
      if (p[i].t=='d' || c==p[i].n2 || c >= maxd || sz<=minp) {
        drawseg(t,c+1);
      }
      else {
        if (p[i].t=='b' || drall) { drawseg(t,c+1); }
        peano(&t,c+1,yf*yF);
      }
    }
}


int main(int argc, char *argv[])
{
  int vm=18,i,c='v'; /* shape */
  FILE *f;
  seg t;
  float u,s=1.0; /* zoom */
  char buf[80],t1[10],t2[10],t3[10];
  int n1,n2,e=0;

  if (argc < 2) { printf(
    " Peano curve generator,  by John Henckel  vers "__DATE__"\n"
    " args:  filename options\n"
    " options: -dn  set max depth [4]\n"
    "    -mn  set minimum size (in pixels) [5]\n"); printf(
    "    -a   draw all levels (recu acts like both) [not]\n"
    "    -zn  zoom factor [1.0]\n"
    "    -vn  video mode 4=cga,14=ega,18=vga [18]\n"); printf(
    "    -sv  shape v=vert,h=horiz,t=trian,s=squar [v]\n"
    " file format:  each line can have one of these\n"
    "    unit n         - size of unit figure [1.0] \n"); printf(
    "    recu x y x y + - do recursion on this segment (flip is + or -)\n"
    "    recu to x y +  - uses x y of previous segment\n"
    "    draw x y x y + - draw this segment but do not do recursion \n"
    "    both x y x y + - both draw and do recursion \n"); printf(
    "    level 0 3      - restricts following segments to levels 0 thru 3\n"
    "    level 3 *      - restricts segments to levels 3 thru the last one.\n"
    "    yfac f         - multiply y values by this at each level [1]\n"
    "    fill x y       - flood fill at point x y.\n"
    " commands can be abbreviated.\n"); printf(
    " Sample file:\n"
    " u 12\n d 6 0 4 4\n b to 10 4\n d to 6 0\n r 0 0 6 0\n r to 12 0\n");
    return 0;
  }
  strcpy(buf,argv[1]);
  if (strrchr(buf,'.')==NULL) strcat(buf,".PNO");
  f = fopen(buf,"r");
  if (f==NULL) return printf("could not open file");
  n=0; n1=0; n2=BIG; u=1;
  while (1) {
    if (fgets(buf,sizeof buf,f)==NULL) break;
    sscanf(buf,"%s %s %s",t1,t2,t3);
    switch (*t1) {
      case 'u': sscanf(t2,"%f",&u); if (u==0) u=1; break;
      case 'y': sscanf(t2,"%f",&yF);  break;
      case 'l':
        n1=0;   if (*t2=='*') n1=BIG; else sscanf(t2,"%d",&n1);
        n2=BIG; if (*t3=='*') n2=BIG; else sscanf(t3,"%d",&n2); break;
      case 'd':
      case 'b':
      case 'r':
        p[n].t=*t1;
        p[n].x1=0; p[n].y1=0;                 /* default values */
        p[n].x2=u; p[n].y2=0; p[n].ys='+';
        if (*t2=='t') {
          sscanf(buf,"%*s %*s %f %f %c",&p[n].x2,&p[n].y2,&p[n].ys);
          if (n>0) { p[n].x1=p[n-1].x2; p[n].y1=p[n-1].y2; }
        }
        else {
          sscanf(buf,"%*s %f %f %f %f %c",
                 &p[n].x1,&p[n].y1,&p[n].x2,&p[n].y2,&p[n].ys);
          p[n].x1/=u; p[n].y1/=u;
        }
        p[n].x2/=u; p[n].y2/=u;
        p[n].n1=n1; p[n].n2=n2;
        ++n; break;
      case 'f':
        p[n].t=*t1;
        sscanf(buf,"%*s %f %f",&p[n].x1,&p[n].y1);
        p[n].x1/=u; p[n].y1/=u;
        p[n].n1=n1; p[n].n2=n2;
        ++n; break;
      default:
        printf("dont understand: %s",buf); e=1;
    }
    if (n>=MAX) { printf("stuff ignored after: %s",buf); e=1; break; }
  }
  fclose(f);

  if (n<1) return printf("file has no segments");
  
  for (i=2; i<argc; ++i) {
    if (*argv[i]!='-') printf("no comprendo '%s'\n",argv[i],e=1);
    else switch(argv[i][1]) {
    case 'a': drall=1; break;
    case 'z': sscanf(argv[i]+2,"%f",&s); break;
    case 'm': sscanf(argv[i]+2,"%d",&minp); break;
    case 'd': sscanf(argv[i]+2,"%d",&maxd); break;
    case 'v': sscanf(argv[i]+2,"%d",&vm); break;
    case 's': c=argv[i][2];
              if (strrchr("hvts",c)==NULL) printf("invalid shape\n",e=1);
              break;
    default: printf("no comprendo '%s'\n",argv[i]); e=1;
    }
  }
  if (e) { printf("\npress enter to continue..."); getchar(); }
  setvmode(vm);
  getvconfig(&vc);

  switch (c) {
  case 'h':
    t.x1=vc.xpixels*(0.5-0.4*s);
    t.x2=vc.xpixels*(0.5+0.4*s);
    t.y1=vc.ypixels*(0.5+0.16*s);
    t.y2=vc.ypixels*(0.5+0.16*s);
    break;
  case 's':
    t.x1=vc.xpixels*(0.5+0.4*s);
    t.x2=vc.xpixels*(0.5-0.4*s);
    t.y1=vc.ypixels*(0.5+0.4*s);
    t.y2=vc.ypixels*(0.5+0.4*s);
    peano(&t,0,1.0);
    t.x1=vc.xpixels*(0.5-0.4*s);
    t.x2=vc.xpixels*(0.5-0.4*s);
    t.y1=vc.ypixels*(0.5+0.4*s);
    t.y2=vc.ypixels*(0.5+0.4*s)-vc.xpixels*s*0.8;
    peano(&t,0,1.0);
    t.x1=vc.xpixels*(0.5-0.4*s);
    t.x2=vc.xpixels*(0.5+0.4*s);
    t.y1=vc.ypixels*(0.5+0.4*s)-vc.xpixels*s*0.8;
    t.y2=vc.ypixels*(0.5+0.4*s)-vc.xpixels*s*0.8;
    peano(&t,0,1.0);
    t.x1=vc.xpixels*(0.5+0.4*s);
    t.x2=vc.xpixels*(0.5+0.4*s);
    t.y1=vc.ypixels*(0.5+0.4*s)-vc.xpixels*s*0.8;
    t.y2=vc.ypixels*(0.5+0.4*s);
    break;
  case 't':
    t.x2=vc.xpixels*(0.5-0.4*s);
    t.x1=vc.xpixels*(0.5+0.4*s);
    t.y1=vc.ypixels*(0.5+0.2*s);
    t.y2=vc.ypixels*(0.5+0.2*s);
    peano(&t,0,1.0);
    t.x1=vc.xpixels*(0.5-0.4*s);
    t.x2=vc.xpixels*0.5;
    t.y1=vc.ypixels*(0.5+0.2*s);
    t.y2=vc.ypixels*(0.5+0.2*s)-vc.xpixels*s*0.8*0.8660254;
    peano(&t,0,1.0);
    t.x1=vc.xpixels*0.5;
    t.x2=vc.xpixels*(0.5+0.4*s);
    t.y1=vc.ypixels*(0.5+0.2*s)-vc.xpixels*s*0.8*0.8660254;
    t.y2=vc.ypixels*(0.5+0.2*s);
    break;
  default:                /* do shape v */
    t.x1=vc.xpixels/2;
    t.x2=vc.xpixels/2;
    t.y1=vc.ypixels*(0.5+0.4*s);
    t.y2=vc.ypixels*(0.5-0.4*s);
  }
  peano(&t,0,1.0);

  while (!kbhit());
  setvmode(DEFAULTMODE);

}
