/*

   John's peano curve generator

   cmdline takes args:
    filename
    maxrecursion
    gmode

*/
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <graphics.h>

/* global types and vars */

#define MAX 50

typedef struct seg {
  float x1,y1,x2,y2;
} seg;

struct vconfig vc;
int maxd = 4;
seg p[MAX]; int d[MAX],n;

#define drawseg(t,f) \
  pen_color(f); \
  move_to(t.x1,t.y1); \
  line_to(t.x2,t.y2)

/*
  This is a recursive function to draw peano curves, the args are
    a - the baseline segment for this level of recursion
    p - the array of segments making the repeatable figure (normalized)
    d - the max depth of each segment in p
    n - number of segments in p
    c - current depth

    if U is the vector form of A, and V is perpendicular to U,
    the segment T = A + px U + py V.
*/
void peano(seg *a, int c)
{
  seg t;
  int i;
  float dx,dy;

  for (i=0; i<n; ++i) {
    dx = a->x2-a->x1;
    dy = a->y2-a->y1;
    t.x1 = a->x1 + p[i].x1*dx + p[i].y1*dy;
    t.y1 = a->y1 - p[i].y1*dx + p[i].x1*dy;
    t.x2 = a->x1 + p[i].x2*dx + p[i].y2*dy;
    t.y2 = a->y1 - p[i].y2*dx + p[i].x2*dy;
    if (d[i] <= c || c >= maxd) {
      drawseg(t,c+1);
    }
    else {
      peano(&t,c+1);
    }
  }
}


int main(int argc, char *argv[])
{
  int vm=CGA_320,c='v';
  FILE *f;
  seg t;
  float s=0.9;

  if (argc < 2) return  printf(
    " Peano curve generator,  by John Henckel\n"
    " args:  filename maxd gmode shape scale (last are optional)\n"
    " filename should contain the unit figure, one segment per line, e.g.\n"
    "   x1 y1 x2 y2 depth \n"
    "    0  0  0 .3 1 \n"
    "    0 .3  1 .5 10\n"
    "    0 .3  1 -0.5 10\n");

  f = fopen(argv[1],"r");
  for (n=0; n<MAX; ++n)
    if (fscanf(f,"%f %f %f %f %d",
         &p[n].x1,&p[n].y1,&p[n].x2,&p[n].y2,&d[n]) != 5)
      break;
  fclose(f);

  if (argc > 2) sscanf(argv[2],"%d",&maxd);
  if (argc > 3) sscanf(argv[3],"%d",&vm);
  if (argc > 4) c=argv[4][0];
  if (argc > 5) sscanf(argv[5],"%f",&s);

  setvmode(vm);
  getvconfig(&vc);

  switch (c) {
  case 'h':
    t.x1=vc.xpixels/10;
    t.x2=vc.xpixels*s;
    t.y1=vc.ypixels/2;
    t.y2=vc.ypixels/2;
    break;
  case 'v':
    t.x1=vc.xpixels/2;
    t.x2=vc.xpixels/2;
    t.y1=vc.ypixels*s;
    t.y2=vc.ypixels/10;
    break;
  case 't':
    t.x1=vc.xpixels*1/2;
    t.x2=vc.xpixels*s;
    t.y1=vc.ypixels*(1-s);
    t.y2=vc.ypixels*s;
    peano(&t,0);
    t.x1=vc.xpixels*s;
    t.x2=vc.xpixels*(1-s);
    t.y1=vc.ypixels*s;
    t.y2=vc.ypixels*s;
    peano(&t,0);
    t.x1=vc.xpixels*(1-s);
    t.x2=vc.xpixels*1/2;
    t.y1=vc.ypixels*s;
    t.y2=vc.ypixels*(1-s);
    break;
  default: return printf("shape must be t,v,h,s\n");
  }
  peano(&t,0);

  while (!kbhit());
  setvmode(DEFAULTMODE);

}
